#include <iostream.h>
#include <stdarg.h>
#include <string.h>

#include "dynstrlb.h"

template <class T>
short compare(T in1, T in2)
{
	short stat=0;
	if (in1 == in2)
		stat = 0;
	else
		stat = (in1 > in2) ? 1 : -1;
	return stat;
}

short compare(char * in1, char *in2)
{
	return strcmp(in1,in2);
}

template <class T> class List;

template <class T>
class Node {
   T data;
   Node *next;
   Node *prev;
public:
   Node();
   ~Node();
   Node(T &indata);
   void connect(Node &other);
   friend class List<T>;
   friend ostream &operator << (ostream &stream, Node &obj);
   friend ostream &operator << (ostream &stream, List<T> &obj);
};

template <class T>
Node<T>::Node() { next = prev = 0; }

template <class T>
Node<T>::~Node() { next = prev = 0; }

template <class T>
Node<T>::Node(T &indata)
{
   next = prev = 0;
   data = indata;
}

template <class T>
void Node<T>::connect(Node<T> &other)
{
   next = &other;
   other.prev = this;
}

template <class T>
ostream &operator << (ostream &stream, Node<T> &obj)
{
   stream << obj.data;
   return stream;
}

template <class T>
class List {
   Node<T> *head;
   Node<T> *tail;
   int cnt;
public:
   	List(); 
   	~List();
	List(int numargs, ...);	
   	void insert(T &indata);
	void remove(T &indata);
	T remove(void); // remove first node in list.
   	friend ostream &operator << (ostream &stream, List &obj);
};

template <class T>
List<T>::List() { head = tail = 0;
	          		cnt = 0; }
	          		
template <class T>
List<T>::~List() 
{
   	Node<T> *traverse=0;
   	for (traverse=tail; traverse; traverse = traverse->prev)
      	delete traverse;
}

template <class T>
List<T>::List(int numargs, ...)
{
	va_list ap;
  	va_start(ap,numargs);
  	head = tail = 0;
   
  	for  (int i=0; i < numargs; i++)
  	{
   	 	 insert(va_arg(ap, T));
  	}
   	va_end(ap);
}


template <class T>
void List<T>::insert(T &indata)
{
   Node<T> *np = new Node<T>(indata);

   cnt++;
   
   if (tail)
   {
      tail->connect(*np);
      tail = np;
   }
   else
   {
      head = np;
      tail = np;
   }
}

template <class T>
void List<T>::remove(T &indata)
{
   Node<T> *traverse=0;
   Node<T> *prev=0;

   for (traverse=head; traverse; prev=traverse,traverse=traverse->next)
   {
		if (!compare(traverse->data,indata))
      	{
      		// match!
      		cnt--;
      		if (prev)
      		{
      			prev->next = traverse->next;
      			(traverse->next)->prev = prev;
      			delete traverse;
      			break;
      		}
      		else
      		{
      			// first node on list
      			head = traverse->next;
      			(traverse->next)->prev = 0;
      			delete traverse;
      			break;
      		}
      	}
   }
}

template <class T>
ostream &operator << (ostream &stream, List<T> &obj)
{
   Node<T> *traverse=0;

   for (traverse=obj.head; traverse; traverse=traverse->next)
   {
      stream << *traverse << endl;
   }
   return stream;
}

template <class T>
T List<T>::remove(void)
{
      T out;
      Node <T> *tmp;
      
      if (cnt)
      {
      	cnt--;
      	tmp = head;
      	head = tmp->next;
      	tmp->next->prev = 0;
      	out = tmp->data;
      	if (!cnt)
      		tail = 0;
      	delete tmp;
      }
      return out;
}

#ifdef __SC__
#pragma template_access public
#pragma template Node<int>
#pragma template Node<double>
#pragma template Node<String>
#pragma template List<int>
#pragma template List<double>
#pragma template List<String>
#pragma template operator <<(ostream&,Node<double>&)
#pragma template operator <<(ostream&,List<double>&)
#pragma template operator <<(ostream&,Node<int>&)
#pragma template operator <<(ostream&,List<int>&)
#pragma template operator <<(ostream&,Node<String>&)
#pragma template operator <<(ostream&,List<String>&)
#pragma template compare(int,int);
#pragma template compare(double,double);
#pragma template compare(String,String);
#endif

int main()
{
	List<int> ilist(5,1,2,3,4,5);
	int tst;

  	cout << "ilist: " << endl;
  	cout << ilist;
	cout << endl;
	
	tst = 3;
	ilist.remove(tst);
	cout << "After removing 3, ilist:" << endl;
	cout << ilist;
	cout << endl;
	 	
 	List<String> slist;
 	
 	slist.insert(String("Mike"));
 	slist.insert(String("Frank"));
 	slist.insert(String("Lynne"));
 	slist.insert(String("CJ"));
	cout << "slist: " << endl;
 	cout << slist;
	cout << endl;
	
	slist.remove(String("Lynne"));
	cout << "After removing 'Lynne', slist:" << endl;
	cout << slist;

	return 0;
}
