template <class T>
short compare(T in1, T in2)
{
   if (in1 == in2)
      return(0);
   else
      return( (in1 > in2)? 1 : -1);
}

short compare(char *in1, char *in2)
{
   return strcmp(in1,in2);
}

template <class T>
class Node {
   T data;
   Node *next;
   Node *prev;
public:
   Node();
   ~Node();
   Node(T &indata);
   void connect(Node &other);
   T getData();
   friend class List<T>;
   friend ostream &operator << (ostream &stream, Node &obj);
   friend ostream &operator << (ostream &stream, List<T> &obj);
};

template <class T>
Node<T>::Node() { next = prev = 0; }

template <class T>
Node<T>::~Node() { next = prev = 0; }

template <class T>
Node<T>::Node(T &indata)
{
   next = prev = 0;
   data = indata;
}

template <class T>
void Node<T>::connect(Node<T> &other)
{
   next = &other;
   other.prev = this;
}

template <class T>
T Node<T>::getData()
{
	return data;
}

template <class T>
ostream &operator << (ostream &stream, Node<T> &obj)
{
   stream << obj.data;
   return stream;
}

template <class T>
class List {
   Node<T> *head;
   Node<T> *tail;
   int cnt;
public:
	List();
	~List();
	List(int numargs, ...);
	void insert(T &indata);
	void remove(T &indata);
	T *get(T &indata);
	friend ostream &operator << (ostream &stream, List<T> &obj);
};

template <class T>
List<T>::List() { head = tail = 0;
               cnt = 0; }

template <class T>
List<T>::~List()
{
   Node<T> *traverse=0;
   for (traverse=tail; traverse; traverse = traverse->prev)
      delete traverse;
}

template <class T>
List<T>::List(int numargs, ...)
{
   va_list ap;
   va_start(ap,numargs);
   head = tail = 0;
   
   for  (int i=0; i < numargs; i++)
   {
      insert(va_arg(ap, T));
   }
   va_end(ap);
}

template <class T>
void List<T>::insert(T &indata)
{
   Node<T> *np = new Node<T>(indata);

   cnt++;
   
   if (tail)
   {
      tail->connect(*np);
      tail = np;
   }
   else
   {
      head = np;
      tail = np;
   }
}

template <class T>
void List<T>::remove(T &indata)
{
   Node<T> *traverse=0,*prev=0;

   for (traverse=head; traverse; prev=traverse,traverse=traverse->next)
   {
      if (!compare(traverse->data,indata))
      {
      // Match! Rearrange links     
      if (prev)
      {
         prev->next = traverse->next;
         traverse->next->prev = prev;
         delete traverse;         
      }
      else
      {
         // head of list
         head->next = traverse->next;
         traverse->next->prev = 0;
         delete traverse;
      }
      break;
      }
   }
}

template <class T>
T *List<T>::get(T &indata)
{
   Node<T> *traverse=0;
   T *p=0;

   for (traverse=head; traverse; traverse=traverse->next)
   {
      if (!compare(traverse->data,indata))
      {
	      // Match! get data
	      p = new T;
	      *p = traverse->data;
	      break;    
      }
   }
   return p;
}

template <class T>
ostream &operator << (ostream &stream, List<T> &obj)
{
   Node<T> *traverse=0;

   for (traverse=obj.head; traverse; traverse=traverse->next)
   {
      stream << *traverse << endl;
   }
   return stream;
}


