// safestr.cpp
#include <iostream.h>
#include <iomanip.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>

const int MAX = 255;

class String {
	protected: 
		char _storage[MAX+1];
		int _sz;
	public:
		String() { _sz = 0; }
		String(char *instr)
		{
			if (instr)
			{
				_sz = strlen(instr);
				assert(_sz < MAX);
				strcpy(_storage,instr);
			}
			else
				_sz = 0;
		}
		
		~String() { _sz = 0; }
		
		// overload the [] for bounds checking
		char &operator [](int index)
		{
			assert(index < _sz);
			assert(index >= 0);
			return _storage[index];
		}
		
		// overload conversion to char * for
		// passing String class to a function
		operator char *()
		{	return _storage; }
		
		// overload stream inserter for 
		// printing string class to cout
		friend ostream &operator << (ostream &stream, String &inobj);
		
		// create an oldcopy function to
		// demonstrate the safety of our class
		void oldcopy(char *instr);
		
		// overload the assignment operator, this
		// would replace the "oldcopy" (hence its name)
		// assignment is more object-oriented
		String &operator = (String &inobj);
		
		// also need to overload the assignment to 
		// accept a regular C++ string
		String &operator = (char *instr);
};

ostream &operator << (ostream &stream, String &inobj)
{
	stream << inobj._storage;
	return stream;
}

void String::oldcopy(char *instr)
{
	int len=0,i=0;
	char *sptr = _storage;
	
	if (instr)
	{
		len = strlen(instr);
		assert(len < MAX);
		_sz = len;
		while ( (*sptr++ = *instr++) != '\0');
	}
}

String &String::operator = (String &inobj)
{
	int i=0;
	char *sptr = _storage;
	char *instr = inobj._storage;
	
	_sz = inobj._sz;
	while ( (*sptr++ = *instr++) != '\0');
	
	return *this;
}	

String &String::operator = (char *instr)
{
	int len=0,i=0;
	char *sptr = _storage;
	
	if (instr)
	{
		len = strlen(instr);
		assert(len < MAX);
		_sz = len;
		while ( (*sptr++ = *instr++) != '\0');
	}

	return *this;
}

void main()
{
	String newstr("Mary");
	char oldstr[] = "John";
	
	cout << "newstr holds <" << newstr << ">" << endl;
	cout << "oldstr holds <" << oldstr << ">" << endl;
	
	// test overrun
	newstr.oldcopy("This is too long");
	cout << "newstr holds <" << newstr << ">" << endl;

	// test assignment	
	newstr = oldstr;
	cout << "newstr holds <" << newstr << ">" << endl;
}
