// dynscore.cpp
#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include <stdarg.h>
#include <new.h>

void exhausted()
{
	cout << "Free store exhausted!" << endl;
	exit(0);
}

class iArray {
	protected:
		int *_storage;
		int _sz, max;
		void grow_array()
		{
			int *tmp=0;
			
			// expand the array
			tmp = new int[max + 10];
			memcpy(tmp,_storage,(sizeof(int) * max));
			delete _storage;
			_storage = tmp;
			max += 10;
		}					
	public:
		iArray() { _sz = 0; max = 0;
				   _storage = 0; }
		iArray(int size)
		{
			_storage = new int[size];
			max = size;
			_sz = 0;
		}
		
		~iArray() { }
		
		int &operator [](int index)
		{
			assert(index >= 0);
			assert(index < _sz);
			return _storage[index];
		}
		
		operator int *()
		{ 	return _storage; }
		
		int bytes()
		{ 	return sizeof(int) * _sz; }
		
		int len()
		{	return _sz; }
		
		void fill(int numelements, ...);
		void print();
};

void iArray::fill(int numelements, ...)
{
	va_list argptr;
	int i=0;
	va_start(argptr, numelements);
	
	for (i=0; i < numelements; i++)
	{
		if (i < numelements)
			_storage[i] = va_arg(argptr, int);
		else
			_storage[i] = 0;
	}
}

void iArray::print()
{
	int i=0;
	cout << "Array Contents:" << endl;
	for (i=0; i < _sz; i++)
	{
		if (i && (!(i%5)))
			cout << endl;
		cout << _storage[i] << " ";
	}
	cout << endl;
}

// our test class will inherit the attributes of the
// iArray class.

class test : public iArray {
	protected:
		int num_students;
		int high,low,sum, average;
	public:
		test()
		{	_sz = num_students = high = sum = average = 0; 
		    low=100;}
		
		~test()
		{	_sz = num_students = high = sum = average = 0; 
		    low = 100;}

		void get_scores();
		int average_score();
		int high_score();
		int low_score();
		void sort();
		void print_stats();
};

void test::get_scores()
{
	int entered_score;
	int i=0,j=0,good_input=0, bad_input=0;
	int score_cnt=0;
	char user_input[256];

	// explain to the user what the program will do.
	cout << "This program will allow you to enter up to 100 test scores.\n";
	cout << "Once scores are entered, I will sort the scores and \n";
	cout << "calculate the average, high and low score.\n";
	
	// we will loop until the user types a -1
	while (entered_score != -1.0)
	{

	  good_input = 0; /* set to false */

	  /* it is a good practice to error check all user input.
	  This will help prevent GIGO! One way to error check
	  user input is to allow him to enter characters and
	  then check the characters and convert as necessary. */
	  while (!good_input)
		{
			bad_input = 0;
			cout << "Enter a score (-1 to stop): ";
			// here we scan a character string into the character array user_input
			cin >> user_input;

			/* I will check all the characters in the
			   string for the occurence of any alpha or control char.
			  **** Notice the end condition of the for loop, 
			  *(user_input + i).  This is our pointer method for accessing
			  array element i in the user_input array.  */
			for (i = 0; *(user_input+i); i++)
			{
				// if the character a letter or cntrl character this is bad input
				if ( (isalpha(*(user_input+i))) || (iscntrl(*(user_input+i))) )
					bad_input = 1;
				if (bad_input) break;
			}
			
			if (!bad_input)
			{
				// translate the string to an integer
				entered_score = atoi(user_input);
				// error check integer
				if ( ((entered_score>0) && (entered_score<=100)) ||
				      (entered_score == -1) )
				      good_input = 1;
				else
					  good_input = 0;
			}
			else
			{
				// print an error an try again
				cout << "Incorrect input. Try again (-1 to quit).\n";
				good_input = 0;
			}
		} /* while not good input */
		
		if (entered_score != -1)
		{
			/* now that we have good input, add it to our array. */
			if (score_cnt >= max)
				grow_array();
			
			_storage[score_cnt] = entered_score;
			score_cnt++; _sz++;
		}
				
	} /* while not -1 or a 100 scores entered */	
}

int test::average_score()
{
	int i;
	
	for (i = 0; i < _sz; i++)
	{
		sum += *(_storage+i);
	}
	
	average = sum/_sz;
	return average;
}

int test::high_score()
{
	int i;
	
	for (i = 0; i < _sz; i++)
	{
		if ( *(_storage+i) > high)
			high = *(_storage+i);				
	}
	
	return high;
}

int test::low_score()
{
	int i;
	
	for (i = 0; i < _sz; i++)
	{
		if ( *(_storage+i) < low)
			low = *(_storage+i);				
	}
	
	return low;
}

void test::sort()
{
	int i,j,temp;
	
	/* bubble sort is the simplest and least efficient sort; however,
	   in our non-data intensive example we won't have to worry about
	   efficiency. */
	for (i = 1; i < _sz; ++i)
	{
		for (j = _sz - 1; j >= i; --j)
		{
			if ( *(_storage + j-1) > *(_storage+j) )
			{
				temp = *(_storage + j - 1);
				*(_storage + j - 1) = *(_storage + j);
				*(_storage + j) = temp;
			} /* end if */
		} /* for */
	} /* for */
}

void test::print_stats()
{
	int i;
	assert(_sz > 1);
	// print the results
	cout << "Number of scores entered was " << _sz << endl;
	sort();
	cout << "The sorted list of scores:\n";
	print();
	cout << "The high score was " << high_score() << endl;
	cout << "The low score was " << low_score() << endl;
	cout << "The average score was " << average_score() << endl;	
}

void main()
{	
	test mytest;
	set_new_handler(exhausted);
	mytest.get_scores();
	mytest.print_stats();
}
