// hyperdct.cpp
#include <iostream.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <new.h>

#include "hyperdct.h"
#include "menu.h"

void exhausted()
{
	cout << "Free store is exhausted." << endl;
}

HyperLink::HyperLink()
{
	_links = 0;
	hyper_cnt = 0;
	hyper_max = 0;
}

HyperLink::~HyperLink()
{
	delete _links;
}

void HyperLink::addLink(int cell)
{
	int *tmp;
	
	if (hyper_cnt == hyper_max)
	{
		// expand
		tmp = new int[hyper_max + 10];
		memcpy(tmp,_links,sizeof(int) * hyper_max);
		delete _links;
		_links = tmp;
		hyper_max += 10;
	}
	
	_links[hyper_cnt] = cell;
	hyper_cnt++;
}

int HyperLink::numLinks()
{
	return hyper_cnt;
}

int &HyperLink::operator [](int index)
{
	assert(index >= 0);
	assert(index < hyper_cnt);
	return _links[index];
}

dictionary_entry::dictionary_entry()
{
	int i;
	definition = 0;
	line_cnt = 0;
	line_max = 0;
}

dictionary_entry::~dictionary_entry()
{
	delete [] definition;
}

String dictionary_entry::get_keyword()
{
	return keyword;
}

short dictionary_entry::word_exists(String &word)
{
	int i=0, result = 0;
	
	for (i=0; i < line_cnt; i++)
	{	
		if (definition[i].Strindex((char *)word) >= 0)
			result = 1;
			
		if (result) break;
	}
	
	return result;
}

void dictionary_entry::LinkTo(int cousin)
{
	connect.addLink(cousin);
}

ostream &operator << (ostream &stream, dictionary_entry &inobj)
{
	int i=0;
	stream << "Keyword: " << inobj.keyword << endl;
	for (i=0; i < inobj.line_cnt; i++)
		stream << "#" << i << ": " << inobj.definition[i] << endl;
				
	return stream;
}

istream &operator >> (istream &stream, dictionary_entry &inobj)
{
	int i=0, done=0,j=0;
	String *tmp = 0;
	cout << "Enter Keyword: ";
	stream >> inobj.keyword;
	stream.ignore(2,'\n');
	
	cout << "Enter definition (empty line to enter):" << endl;
	while (!done)
	{
		if (i >= inobj.line_max)
		{
			tmp = new String[inobj.line_max + 10];
			
			for (j=0; j < inobj.line_max; j++)
				tmp[j] = inobj.definition[j];
				
			delete [] inobj.definition;
			inobj.definition = tmp;
			inobj.line_max += 10;
		}
		
		cout << "#" << i << ": ";
		inobj.definition[i].getline(stream);

		if (inobj.definition[i].getlen() > 0)
			i++;
		else
			done = 1;
	}
	inobj.line_cnt = i;
	
	return stream;
}

dictionary::dictionary()
{
	listing = 0;
	entry_count = 0;
	entry_max = 0;
}

dictionary::~dictionary()
{
	delete [] listing;
}

void dictionary::add_entry()
{
	int i=0,j=0;
	dictionary_entry *tmp = 0;
	if (entry_count >= entry_max)
	{
		tmp = new dictionary_entry[entry_max + 10];
		
		for (j=0; j < entry_max; j++)
			tmp[j] = listing[j];
			
		delete [] listing;
		listing = tmp;
		entry_max += 10;	
	}
	
	cin >> listing[entry_count];
	
	// *** add hyperlinks ***
	for (i=0; i < entry_count; i++)
	{
		if (listing[i].word_exists(listing[entry_count].get_keyword()))
		{
			listing[i].LinkTo(entry_count);
			listing[entry_count].LinkTo(i);
		}
	}
	
	entry_count++;
}

void dictionary::delete_entry()
{
	cout << "Stub code" << endl;
}

void dictionary::list_entries()
{
	int i=0;
	cout << "Dictionary Listing:" << endl;
	for (i=0; i < entry_count; i++)
		cout << "Entry #" << i << ": "
		     << listing[i].get_keyword() << endl;
	cout << "End of Dictionary Listing." << endl;
}

void dictionary::display_entry(int index)
{
	int j=0;
	cout << listing[index];
	if (listing[index].connect.numLinks())
	{
		cout << "<<<- Hyper Links ->>>" << endl;
		cout << "0) Return to Main Menu" << endl;
		for (j=0; j < listing[index].connect.numLinks(); j++)
		{
			cout << j+1 << ") " << "Display "
			     << listing[listing[index].connect[j]].get_keyword()
			     << endl;
		}
	}
}

void dictionary::find_entry()
{
	int i=0,j=0,entry=0,thechoice=0;
	int good=0,numlinks=0,done=0;
	String key;
	cout << "Enter keyword to find: ";
	cin >> key;
	for (i=0; i < entry_count; i++)
	{
		if (key == listing[i].get_keyword())
		{
			cout << "Entry # " << i << endl;
			entry = i;
			while (!done)
			{
				display_entry(entry);
				numlinks = listing[entry].connect.numLinks();
				if (numlinks)
				{
					good = 0;
					while (!good)
					{
						cout << "Choice : ";
						cin >> thechoice; 
						if (thechoice == 0)
						{
							done = 1;
							good = 1;
							break;
						}
						
						if (thechoice > 0 && thechoice <= numlinks)
						{
							good = 1;
							entry = listing[entry].connect[thechoice - 1];
						}	
					}
				}
				else
					done = 1;
			}
			break;
		}
	}

	if (i== entry_count)
	{
		cout << "No match for " << key << endl;
	}
}

void main()
{
	int done=0;
	dictionary webster;
	menu todo(5,1,0,"<<<< Hyper-Webster >>>>",
		            "1) enter a definition.",
				    "2) list all entries.",
		            "3) display an entry.",
		            "4) exit.");
	set_new_handler(exhausted);
	
	while (!done)
	{
		cout << todo;
		switch (todo.menu_choice()) {
			case 1:
					webster.add_entry();
				break;
			case 2:
					webster.list_entries();
				break;
			case 3:
					webster.find_entry();
				break;
			case 4:
				done=1;
				break;
			default:
				cout << "Invalid Choice. Try again." << endl;
		}	
	}
}

